package com.example.smartlab.viewmodel

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.smartlab.api.News
import com.example.smartlab.api.Repository
import com.example.smartlab.api.Result
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch

/** создадим ViewModel, в которой будут соданы переменные для получения данных
и в дальнейшим их использование в UI и метод подключения к API**/

/** назначение ViewModel хранить данные, связанные с пользовательским интерфейсом, и управлять ими
, его основная функция заключается в поддержании целостности и позволяет обслуживать данные во время изменений
конфигурации, таких как поворот экрана.
Любое изменение конфигурации на устройствах Android приводит к воссозданию всей активности приложения.*/

//принимает репозиторий, возвращает вьюмодель
class ViewModelMain(private val repository: Repository): ViewModel() {
    /** Поток изменяемых состояний (MutableStateFlow) - это поток, который может быть как собран, так и преобразован,
    и у него есть изменяемое состояние, которое может быть изменено сборщиком. Это означает,
    что вы можете использовать поток изменяемых состояний для представления состояния представления
    и обновления его на основе пользовательского ввода или других событий.*/

    /**
    StateFlow в Kotlin ведёт себя так же, как LiveData, за исключением того,
    что LiveData имеет осведомлённость о жизненном цикле и не требует начального значения.
     *
    Чтобы создать поток StateFlow, нужно создать экземпляр MutableStateFlow с обязательным начальным значением.
    Затем вызвать asStateFlow() на этом экземпляре, чтобы преобразовать его в StateFlow,
    из которого можно собирать изменения состояния.*/

    //Каналы (Channel) позволяют передавать потоки данных
    private val _showErrorToastChannel = Channel<Boolean>()
    val showErrorToastChannel = _showErrorToastChannel.receiveAsFlow()
    fun sendCodeToEmail(email: String){
        //сфера применения vm
        //запустить
        viewModelScope.launch {
            repository.sendCodeToEmail(email).collect(){
                when(it){
                    /** Instanceof (is) - это оператор на языке java, здесь, в Kotlin, мы использовали is и !
                    это ключевые слова для выполнения операций, таких как instanceof,
                    то есть тип свойства доступен или нет, это функция, подобная способу проверки типа,
                    для проверки типа конкретного экземпляра или других различных переменных во время выполнения,
                    для разделения рабочего процесса.*/
                    is Result.Error ->{
                        _showErrorToastChannel.send(true)
                    }
                    is Result.Success ->{
                        _showErrorToastChannel.send(false)
                    }
                    //проверка на тип и соответствующее заполнение true/false
                }
            }
        }
    }

    fun SignIn(email: String, code: String){
        viewModelScope.launch {
            repository.signIn(email, code).collect{
                when(it){
                    is Result.Error ->{
                        _showErrorToastChannel.send(true)
                        //hint
                    }
                    is Result.Success ->{
                        _showErrorToastChannel.send(false)

                    }
                }
            }
        }
    }

    private val _news = MutableStateFlow<List<News>>(emptyList())
    val news = _news.asStateFlow()
    fun getNews(){
        viewModelScope.launch {
            repository.getNews().collect { result ->
                when(result) {
                    is Result.Error -> {
                        _showErrorToastChannel.send(true)
                    }
                    is Result.Success -> {
                        result.data?.let { new ->
                            _news.update {new}
                        }
                    }
                }
            }
        }
    }


}
